<#
.SYNOPSIS

This script file is designed to deploy PackageManagementService to Azure.
.DESCRIPTION

The purpose of this script is to deploy PackageManagementService from WDP to Azure.
The deployment process consist of 2 steps:
1. Deployment of the WDP into Azure storage
2. Deployment of the WebApp from WDP
.EXAMPLE

PS> .\AzureDeploy.ps1 -subscriptionId "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx" -wdpPackagePath "C:\deployment-package\PackageManagementServicePackage.x.y.z.rev-ddmmyy.azure-wdp.zip" -azureSecurityKey "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx" -azureClientID "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx" -azureTenantID "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"
.EXAMPLE

PS> .\AzureDeploy.ps1 -subscriptionId "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx" -resourceGropuName "myResourceGroup" -resourceGroupLocation "North Europe" -wdpPackagePath "C:\deployment-package\PackageManagementServicePackage.x.y.z.rev-ddmmyy.azure-wdp.zip" -storageContainerName "myStorageContainer" -azureSecurityKey "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx" -azureClientID "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx" -azureTenantID "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"
.PARAMETER subscriptionId

The id of Azure subscription.
.PARAMETER resourceGroupName

The name of Azure resource group.
Default value: sitecorepackagemanagamentservice
.PARAMETER resourceGroupLocation

The location of Azure resource group.
Default value: North Europe
.PARAMETER wdpPackagePath

The path to the WDP .zip archive to deploy service from.
.PARAMETER storageContainerName

The name of Azure storage container.
Default value: packagemanagementservicestorage
.PARAMETER azureSecurityKey

The Azure security key.
.PARAMETER azureClientID

The Azure client ID.
.PARAMETER azureTenantID

The Azure tenant ID.
#>
param(
    [Parameter(Mandatory = $True)]
    [string]
    $subscriptionId,

    [string]
    $resourceGroupName = "sitecorepackagemanagamentservice",

    [string]
    $resourceGroupLocation = "North Europe",

    [Parameter(Mandatory = $True)]
    [string]
    $wdpPackagePath,

    [string]
    $storageContainerName = "packagemanagementservicestorage",

    [parameter(Mandatory = $true)]
    [string]
    $azureSecurityKey,

    [parameter(Mandatory = $true)]
    [string]
    $azureClientID,

    [parameter(Mandatory = $true)]
    [string]
    $azureTenantID
)

Write-Host -BackgroundColor Cyan "Deploying PackageManagementService package to Azure storage.";

$secpasswd = ConvertTo-SecureString $azureSecurityKey -AsPlainText -Force;
$mycreds = New-Object System.Management.Automation.PSCredential ($azureClientID, $secpasswd);
Login-AzureRmAccount -ServicePrincipal -Tenant $azureTenantID -Credential $mycreds;

Select-AzureRmSubscription -SubscriptionId $subscriptionId

# Check whether Azure resource group exist, if not it will be created
$resourceGroup = Get-AzureRmResourceGroup -Name $resourceGroupName -ev notPresent -ea 0;
if (!$resourceGroup) {
    Write-Output("Creating resource group '$resourceGroupName' in location '$resourceGroupLocation'.");
    New-AzureRmResourceGroup -Name $resourceGroupName -Location $resourceGroupLocation
}
else {
    Write-Output("Using existing resource group '$resourceGroupName'.");
}

# Deploy Azure storage account from ARM template
New-AzureRmResourceGroupDeployment -ResourceGroupName $resourceGroupName -TemplateFile ".\templates\storageDeploy.template.json" `
    -TemplateParameterFile ".\templates\storageDeploy.parameters.json";

$storageAccountName = (Get-AzureRmResourceGroupDeployment -ResourceGroupName $resourceGroupName -Name "storageDeploy.template").Outputs.storageAccountName.value;
$storageAccount = Get-AzureRmStorageAccount -ResourceGroupName $resourceGroupName -AccountName $storageAccountName;
$storageAccountContext = $storageAccount.Context;

# Check whether Azure storage contaner exist, if not it will be created
$storageContainer = Get-AzureStorageContainer -Name $storageContainerName -Context $storageAccountContext -ev notPresent -ea 0;
if (!$storageContainer) {
    Write-Output("Creating storage container '$storageContainerName' in storage account '$storageAccountName'.");
    New-AzureStorageContainer -Name $storageContainerName -Context $storageAccountContext -Permission blob;
}
else {
    Write-Output("Using existing storage container '$storageContainerName' in storage account '$storageAccountName'.");
}

# Deploy WDP package to storage container. If the file with same name already exist in container, it will be overwritten    
$wdpPackageName = Split-Path $wdpPackagePath.ToLower() -Leaf

Set-AzureStorageBlobContent -File $wdpPackagePath.ToLower() -Container $storageContainerName -Blob $wdpPackageName -Context $storageAccountContext -Force

$azureBlobSASToken = New-AzureStorageBlobSASToken -Context $storageAccountContext -Container $storageContainerName -Blob $wdpPackageName -Permission rwd;

Write-Host -BackgroundColor Cyan "Deploying PackageManagementService package from Azure storage to WebApp";

# Deploy Web App from ARM template 
New-AzureRmResourceGroupDeployment -ResourceGroupName $resourceGroupName -TemplateFile ".\templates\serviceDeploy.template.json" `
    -TemplateParameterFile ".\templates\serviceDeploy.parameters.json" `
    -msDeployPackageUrl (ConvertTo-SecureString "https://$storageAccountName.blob.core.windows.net/$storageContainerName/$wdpPackageName$azureBlobSASToken" -AsPlainText -Force)