param($installPath, $toolsPath, $package, $project)

# NOTICE! This file is executed during during both installation and update.

# Find solution file location
$rootPath = (Get-Item $installPath).Parent.Parent.FullName
Write-Host "Script environment: RootPath = '$rootPath', Project='$($project.Name)'"

# Rewrite the file with actual settings as it isn't supposed to be modified by consumers.
# Usually, the fie shouldn't exist as the previous Uninstall.ps1 should remove it.
$qualitySettingsFilePath = (Join-Path $rootPath "Sitecore.CodeQuality.DotSettings")
Copy-Item -Force (Join-Path $toolsPath "Sitecore.CodeQuality.DotSettings") $qualitySettingsFilePath
Write-Host "Created/Rewritten file: '$qualitySettingsFilePath'"

# Import helper functions to simplify work with DotSettings file.
. (Join-Path $toolsPath "Helpers.ps1")

# Process each solution file
Get-Item $rootPath\*.sln | ForEach-Object {
    # Get the name of the solution file
    $slnName = $_.Name

    # Determine the soure and target paths for the DotSettings file
    $slnSettingsFile = Join-Path $rootPath "$slnName.DotSettings"
    $templateSlnSettingsFile = Join-Path $toolsPath "Solution.DotSettings"

    # If solution specific file doesn't exist - we just create it using the template as a base.
    if(-not (Test-Path $slnSettingsFile)) {
        Copy-Item $templateSlnSettingsFile $slnSettingsFile
        Write-Host "Created file: '$slnSettingsFile'"
        return
    }

    # The solution specific file already exists and we cannot simply override it to not remove team settings.
    # Instead, we need to patch the existing settings file and add references to custom layer if necessary.
    # The %SOLUTION%.DotSettings file is supposed to include an additional layer with actual configuration and might be modified by teams.
    try {
        # Parse template file and existing solution specific file as an xml
        $templateDoc = [xml](Get-Content $templateSlnSettingsFile)
        $existingDoc = [xml](Get-Content $slnSettingsFile)

        # Find all settings nodes in both template and existing DotSettings files
        $templateNodes = Find-ReSharperSettingNodes $templateDoc
        $existingNodes = Find-ReSharperSettingNodes $existingDoc

        # Find the root node for all settings in the existing DotSettings file.
        # We will use this root to insert new setting entries
        $existingDocSettingsRoot = Get-ReSharperSettingsRootNode $existingDoc

        # Iterate over each template setting and insert it to the existing DotSettings file
        foreach($templateNode in $templateNodes) {
            # Find if the template setting already exists in the current DotSettings file.
            # If we found it - remove it as setting value might be changed.
            $existingNode = Find-MatchingReSharperSettingNode $existingNodes $templateNode
            if($existingNode) {
                $existingNode.ParentNode.RemoveChild($existingNode) | Out-Null
            }

            # Insert the setting entry from template DotSettings file to the existing DotSettings file.
            $importedNode = $existingDoc.ImportNode($templateNode, $true)
            $existingDocSettingsRoot.AppendChild($importedNode) | Out-Null
        }

        # Save all the changes to the solution's DotSettings file.
        $existingDoc.Save($slnSettingsFile)
        Write-Host "Patched file: '$slnSettingsFile'"
    } catch {
        Write-Host "Unable to patch the file '$slnSettingsFile'. $($_.Exception.Message)" -ForegroundColor "Red"
    }
}

# Get relative path to tools folder from project folder
$projectDir = (Get-Item $project.FullName).Directory
Push-Location $projectDir
$relativeToolsPath = Resolve-Path -Relative $toolsPath
Pop-Location

# Set Code Analysis ruleset in all configurations
$project.ConfigurationManager | ForEach-Object {
    $_.Properties.Item("CodeAnalysisRuleSet").Value = (Join-Path $relativeToolsPath "Sitecore.ruleset")
    $_.Properties.Item("RunCodeAnalysis").Value = "true"

    Write-Host "RuleSet assigned for: Configuration='$($_.ConfigurationName)' Platform='$($_.PlatformName)'"
}