param($installPath, $toolsPath, $package, $project)

# NOTICE! This file is executed during during both installation and update.

# Find solution file location
$rootPath = (Get-Item $installPath).Parent.Parent.FullName
Write-Host "Script environment: RootPath = '$rootPath', Project='$($project.Name)'"

# Remove the file with actual settings as it isn't supposed to be modified by consumers.
$qualitySettingsFilePath = Join-Path $rootPath "Sitecore.CodeQuality.DotSettings"
if(Test-Path $qualitySettingsFilePath) { 
	Remove-Item $qualitySettingsFilePath 
	Write-Host "Removed file: '$qualitySettingsFilePath'"
}

# Import helper functions to simplify work with DotSettings file.
. (Join-Path $toolsPath "Helpers.ps1")

# Process each solution file
Get-Item $rootPath\*.sln | ForEach-Object {
	# Get the name of the solution file
	$slnName = $_.Name
	
	# Determine the target path for the solution specific DotSettings file
	$slnSettingsFile = Join-Path $rootPath "$slnName.DotSettings"
	$templateSlnSettingsFile = Join-Path $toolsPath "Solution.DotSettings"

	# If solution specifc DotSettings file is missing - just do nothing
	if(-not (Test-Path $slnSettingsFile)) {
		return
	}

	# It's unsafe to remove the solution's DotSettings file, as it might be modified and might contain team-specific settings.
	# Instead we should parse the existing file and remove all the template setting entries we added during the installation.
	# The %SOLUTION%.DotSettings file is supposed to include an additional layer with actual configuration and might be modified by teams.
	try {
		# Parse template file and existing solution specific file as an xml
		$templateDoc = [xml](Get-Content $templateSlnSettingsFile)
        $existingDoc = [xml](Get-Content $slnSettingsFile)

		# Find all settings nodes in both template and existing DotSettings files
        $templateNodes = Find-ReSharperSettingNodes $templateDoc
        $existingNodes = Find-ReSharperSettingNodes $existingDoc

		# Iterate over each template node, try to find the matching node in existing DotSettings file.
		# If the matching node exists - remove it.
		foreach($templateNode in $templateNodes) {
			$existingSettingNode = Find-MatchingReSharperSettingNode $existingNodes $templateNode
			if($existingSettingNode) {
				$existingSettingNode.ParentNode.RemoveChild($existingSettingNode) | Out-Null
			}
		}

		# Save all the changes to the solution's DotSettings file.
		$existingDoc.Save($slnSettingsFile)
		Write-Host "Patched file: '$slnSettingsFile'"
	} catch {
		Write-Host "Unable to patch the file '$slnSettingsFile'. $($_.Exception.Message)" -ForegroundColor "Red"
	}
}